;---------------------------==[ ZERO PONG v3.0 ]==----------------------------

;                              6502 source code

;                                by zero soul
;                             cr_3xs@hotmail.com
;                         http://zerosoul.tripod.com

;originally posted at NESDEV (http://nesdev.parodius.com)

;written for NESASM v2.0 (look for the "Magic Kit" at the NESDEV site)

;define iNES header directives... NESASM will generate the iNES header from
;these.
        .inesprg 1      ;16KB PRG-ROM (code) bank count
        .ineschr 0      ;8KB CHR-ROM / VROM (pattern table) bank count
        .inesmir 0      ;VRAM bank mirroring (unneeded)
        .inesmap 0      ;iNES memory mapping chipset ID# (ie. Mapper #)

        ;just a comment... why doesn't NESASM support other header stuff such
        ;as Save RAM? I don't need it yet, but still...
        ;(of course, if I really needed it, I would .db the iNES header manu-
        ;ally, and assemble it "raw" (sans header))

        .org $c000      ;origin of address counter

;PARAMETERS (memory storage allocation, not part of actual code)

        .zp             ;zero-page section

;All the paddle and ball data are 16-bit (the high-order bytes are used for
;the actual on-screen coordinates when it comes time to set them)-
;.LO indicates the low-order byte, .HI, the high-order byte, for each

PAD.X.LO = $00          ;Paddles' X-coordinate (on far left edge)
PAD.X.HI = $01
PAD.XM.LO = $02         ;Paddle X movement
PAD.XM.HI = $03
BALL.X.LO = $04         ;Ball X-coord
BALL.X.HI = $05
BALL.Y.LO = $06         ;Ball Y-coord (on top edge)
BALL.Y.HI = $07
BALL.XM.LO = $08        ;Ball X movement
BALL.XM.HI = $09
BALL.YM.LO = $0A        ;Ball Y movement
BALL.YM.HI = $0B
TEMP1 = $0C             ;temp storage spaces
TEMP2 = $0D
POINT.LO = $0E          ;pointer value
POINT.HI = $0F
A.READ = $10            ;joystick input ($01 = pressed, $00 = not pressed)
B.READ = $11            ;(only joystick #1 is used)
SEL.READ = $12
STA.READ = $13
UP.READ = $14
DOWN.READ = $15
LEFT.READ = $16
RIGHT.READ = $17
PAD.Y.LO = $18          ;Side Paddles' Y-coordinate (on far top edge)
PAD.Y.HI = $19
PAD.YM.LO = $1A         ;Y-movement
PAD.YM.HI = $1B
SCORE.LO = $1C          ;low digit of score
SCORE.HI = $1D          ;high digit of score
SCORE.MID = $1E         ;middle digit of score
ACCEL.FACT = $1F        ;how much have you accelerated?

;CONTROLS:
;Left/Right move the top/bottom paddles, Up/Down move the side paddles
;hold B to make the paddles slowly gain speed; when you release B, the effect
;will remain and gradually wear off
;press Start to restart when you lose
;press Select+Start to reset at any time

;top/bottom paddles have three tiles L,M,R (left middle right)
;side paddles have three tiles T,C,B (top center bottom)
;PAD1 is bottom paddle, PAD2 is top tile, BALL is the ball (duh)
;PAD3 is the left paddle, PAD4, the right paddle
;X = X-coordinate on left edge
;Y = Y-coordinate on top edge
;T = Tile index #
;S = Status/properties byte
;eg: PAD2L.S is properties byte of left tile of top paddle
;occurs on NES in this order: Y-coordinate, Tile #, Status, X-coordinate
;occurs here in RAM: PAD1L,PAD1M,PAD1R,PAD2L,PAD2M,PAD2R,BALL,PAD3T,PAD3M,
;       PAD3B,PAD4T,PAD4C,PAD4B (ie. bottom, top, ball, left, right)

        .bss            ;RAM ($0200-$07FF) section

PAD1L.Y = $0700         ;this is how it's stored in the 7-page- the data sto-
PAD1L.T = $0701         ;red here is what is actually copied to the PPU (to
PAD1L.S = $0702         ;the screen)
PAD1L.X = $0703
PAD1M.Y = $0704         ;four bytes per sprite (Y,X,tile#,properties (S))
PAD1M.T = $0705
PAD1M.S = $0706
PAD1M.X = $0707
PAD1R.Y = $0708
PAD1R.T = $0709
PAD1R.S = $070A
PAD1R.X = $070B
PAD2L.Y = $070C
PAD2L.T = $070D
PAD2L.S = $070E
PAD2L.X = $070F
PAD2M.Y = $0710
PAD2M.T = $0711
PAD2M.S = $0712
PAD2M.X = $0713
PAD2R.Y = $0714
PAD2R.T = $0715
PAD2R.S = $0716
PAD2R.X = $0717
BALL.Y = $0718
BALL.T = $0719
BALL.S = $071A
BALL.X = $071B
PAD3T.Y = $071C
PAD3T.T = $071D
PAD3T.S = $071E
PAD3T.X = $071F
PAD3C.Y = $0720
PAD3C.T = $0721
PAD3C.S = $0722
PAD3C.X = $0723
PAD3B.Y = $0724
PAD3B.T = $0725
PAD3B.S = $0726
PAD3B.X = $0727
PAD4T.Y = $0728
PAD4T.T = $0729
PAD4T.S = $072A
PAD4T.X = $072B
PAD4C.Y = $072C
PAD4C.T = $072D
PAD4C.S = $072E
PAD4C.X = $072F
PAD4B.Y = $0730
PAD4B.T = $0731
PAD4B.S = $0732
PAD4B.X = $0733

T.SU.Y = $0734          ;japanese title
T.SU.T = $0735
T.SU.S = $0736
T.SU.X = $0737
T.I.Y = $0738
T.I.T = $0739
T.I.S = $073A
T.I.X = $073B
T.RO.Y = $073C
T.RO.T = $073D
T.RO.S = $073E
T.RO.X = $073F
T.HO.Y = $0740
T.HO.T = $0741
T.HO.S = $0742
T.HO.X = $0743
T.N.Y = $0744
T.N.T = $0745
T.N.S = $0746
T.N.X = $0747
T.KU.Y = $0748
T.KU.T = $0749
T.KU.S = $074A
T.KU.X = $074B
T.M1.Y = $074C          ;mark #1 (first ")
T.M1.T = $074D
T.M1.S = $074E
T.M1.X = $074F
T.M2.Y = $0750          ;mark #2 (the little circle)
T.M2.T = $0751
T.M2.S = $0752
T.M2.X = $0753
T.M3.Y = $0754          ;mark #3 (second ")
T.M3.T = $0755
T.M3.S = $0756
T.M3.X = $0757

S1.Y = $0758            ;high digit of score
S1.T = $0759
S1.S = $075A
S1.X = $075B
S2.Y = $075C            ;mid digit of score
S2.T = $075D
S2.S = $075E
S2.X = $075F
S3.Y = $0760            ;low digit of score
S3.T = $0761
S3.S = $0762
S3.X = $0763

;That's all the parameters. Lot of text space but remember, so far not a sin-
;gle byte has been assembled; those were just labels, identifiers; if I use
;one of them as the argument, the assembler will substitute the label with
;the values defined above, eg:
;STA PAD4C.S
;is the same, here, as STA $072E, and will be assembled that way. Einfach!
;however, even though I defined some RAM parts as zero-page, NESASM will still
;return 16-bit values (at least it will use the correct opcode) unless I spe-
;cify zero-page below by placing "<" (sans quotes) before the argument.
;eg: sta TEMP1 returns sta $001C; sta <TEMP1 returns sta $1C, using the zero-
;page addressing mode opcode

        .code           ;Code section (start of actual program)

; EMBEDDED TILES
; 0 = BG color (black); 1 = Dark colour, 2 = Middle colour, 3 = Light colour

tileset:                ;row 1 of tiles (13 ($D) tiles)
        .defchr $00000000,\     ;ends of horizontal paddles
                $00000000,\
                $33333333,\
                $32222231,\
                $12112231,\
                $12332111,\
                $12211000,\
                $11100000
        .defchr $00000000,\     ;middle of horiz. tiles
                $00000000,\
                $33333333,\
                $22211222,\
                $22233222,\
                $11111111,\
                $00000000,\
                $00000000       ;0=black 1=yellow
        .defchr $00000000,\     ;ball
                $00022000,\
                $00232200,\
                $02333210,\
                $02232210,\
                $00222100,\
                $00011000,\
                $00000000
        .defchr $11113300,\     ;top/bottom of vert. paddles
                $12222300,\
                $12312300,\
                $01312300,\
                $01222300,\
                $00122300,\
                $00133300,\
                $00111300
        .defchr $00122300,\     ;middle of vert. paddles
                $00122300,\
                $00122300,\
                $00131300,\
                $00131300,\
                $00122300,\
                $00122300,\
                $00122300
        .defchr $00000000,\     ;katakana "su"
                $13333330,\     ;adding " makes it "zu"
                $00000130,\
                $00000130,\
                $00001300,\
                $00012000,\
                $00121200,\
                $12200120
        .defchr $00000000,\     ;katakana dropped "i"
                $00000000,\     ;placing after "zu" makes "zi"
                $00000000,\     ;(before, it was "ji")
                $00013000,\
                $00130000,\
                $01330000,\
                $12120000,\
                $00120000
        .defchr $00000000,\     ;katakana "ro"
                $00000000,\
                $13333330,\
                $13000130,\
                $13000120,\
                $12000120,\
                $12222220,\
                $00000000
        .defchr $00000000,\     ;katakana "ho"
                $00013000,\     ;adding the circle mark makes it "po"
                $13333333,\
                $00013000,\
                $01313130,\
                $12012012,\
                $00012000,\
                $00122000
        .defchr $00000000,\     ;katakana "n"
                $00000000,\
                $00000000,\
                $13300000,\
                $00000130,\
                $00001300,\
                $00122000,\
                $12200000
        .defchr $00000000,\     ;katakana "ku"
                $01300000,\     ;adding " makes it "gu"
                $01333330,\
                $13000130,\
                $12000130,\
                $00001200,\
                $00012000,\
                $01220000
        .defchr $00000000,\     ;katakana " mark
                $00000000,\
                $00000000,\
                $00000000,\
                $00000000,\
                $00000000,\
                $00001313,\
                $00001212
        .defchr $00000000,\     ;katakana circle mark
                $00000000,\
                $00000000,\
                $00000000,\
                $00000000,\
                $00000130,\
                $00001312,\
                $00000120
                                ;so overall, it's ZI-RO-PO-N-GU
t0:
        .defchr $00000000,\     ;zero
                $01333300,\
                $13301330,\
                $13301330,\
                $12201220,\
                $12201220,\
                $12201220,\
                $01222200
t1:
        .defchr $00000000,\     ;one
                $00133000,\
                $01333000,\
                $00133000,\
                $00122000,\
                $00122000,\
                $00122000,\
                $01222200
t2:
        .defchr $00000000,\     ;two
                $01333300,\
                $13301330,\
                $00001330,\
                $00012200,\
                $00122000,\
                $01220000,\
                $12222220
t3:
        .defchr $00000000,\     ;three
                $01333300,\
                $13301330,\
                $00001330,\
                $00122200,\
                $00001220,\
                $12201220,\
                $01222200
t4:
        .defchr $00000000,\     ;four
                $00013300,\
                $00133300,\
                $01313300,\
                $12012200,\
                $12222220,\
                $00012200,\
                $00012200
t5:
        .defchr $00000000,\     ;five
                $13333330,\
                $13300000,\
                $13300000,\
                $12222200,\
                $00001220,\
                $12201220,\
                $01222200
t6:
        .defchr $00000000,\     ;six
                $01333300,\
                $13301330,\
                $13300000,\
                $12222200,\
                $12201220,\
                $12201220,\
                $01222200
t7:
        .defchr $00000000,\     ;seven
                $13333330,\
                $00001330,\
                $00013300,\
                $00122000,\
                $00122000,\
                $00122000,\
                $00122000
t8:
        .defchr $00000000,\     ;eight
                $01333300,\
                $13301330,\
                $13301330,\
                $01222200,\
                $12201220,\
                $12201220,\
                $01222200
t9:
        .defchr $00000000,\     ;nine
                $01333300,\
                $13301330,\
                $13301330,\
                $01222220,\
                $00001220,\
                $12201220,\
                $01222200

num_pointers:   ;this generates a list of the addresses of each number tile
        .dw t0,t1,t2,t3,t4,t5,t6,t7,t8,t9
        ;check out the end of the "move2" routine and the "do_score" routine
        ;near the end of the program to see how these are used

init_data:              ;I moved it up here for easier access
        .db 24                  ;far left edge of PAD1/2
        .db 16                  ;Y of PAD2
        .db 208                 ;Y of PAD1
        .db 179                 ;far top edge of PAD3/4
        .db 3                   ;X of PAD3
        .db 240                 ;X of PAD4
        .db 0,1,$40,$80,$81,$c0 ;property bytes for all sprites in PAD1/2
        .db 0,1,$80,$40,$41,$c0 ;property bytes for all sprites in PAD3/4
        .db 1,2                 ;tile# PAD1/2: LEFT/RIGHT then MIDDLE
        .db 4,5                 ;tile# PAD3/4: TOP/BOTTOM then CENTER
        .db 128,3,2,128         ;entire ball sprite data

        .db 208                 ;far top edge of japanese title text
        .db 8                   ;far left edge of "
        .db %00000011           ;colour set for all sprites there

        .db 208                 ;far left edge of scores
        .db 8                   ;far top edge of scores
        .db %00000011           ;colour set of scores

main:                   ;this is where it all begins
        sei             ;set interrupt-disable flag
        cld             ;disable un-l33t decimal mode
        ldx #$ff        ;ensure stack pointer points to top of stack
        txs
        jsr palette     ;set the palette
        jsr clr700      ;clear 7-page ($0700-$07FF) before using it
        jsr inits       ;copy initial sprite data and set parameter inits
        jsr cleartiles  ;clear all tile space before using it
        jsr tiles       ;copy embedded tiles to VRAM
        jsr ppuinit     ;initialize/setup the screen display
        jmp body        ;get it ON

cleartiles:                     ;clears out $0000-$0FFF in VRAM
        jsr wait_vblank
        lda #0
        sta $2006
        sta $2006
        ldy #$10
cleartiles1:
        ldx #0
cleartiles2:
        sta $2007
        dex
        bne cleartiles2
        dey
        bne cleartiles1
        rts

wait_vblank:            ;stalls until high bit of $2002 is set (ie. the PPU is
        bit $2002       ;going into VBlank)
        bpl wait_vblank
        rts

           ;     Gold            White            Red            Blue
paldata:   ;[-----1st-----] [-----2nd-----] [-----3rd-----] [-----4th-----]
        .db $0f,$08,$18,$38,$0f,$00,$10,$30,$0f,$06,$16,$26,$0f,$01,$11,$31
palette:
        lda #$3f        ;set VRAM write address ($3F00)
        sta $2006
        lda #0
        sta $2006
        ldy #0          ;reset indexing register
        ldx #$20        ;countdown timer
do_palette:
        lda paldata,y
        sta $2007
        iny
        tya             ;these three lines reset Y to $00 when it reaches $10;
        and #$0f        ;this effectively copies the $10-byte palette array
        tay             ;to VRAM twice (which is what I want)
        dex
        bne do_palette
        rts

clr700:                         ;clears out $700-$7FF
        lda #0
        ldy #0
do_clr700:
        sta $700,y
        iny
        bne do_clr700
        rts

inits:
        lda init_data         ;X-coord. of left tile of paddles
        sta <PAD.X.HI            ;is also the paddle X location in RAM
        sta PAD1L.X
        sta PAD2L.X
        adc #8                  ;X-coord. of middle tile
        sta PAD1M.X
        sta PAD2M.X
        adc #8                  ;X-coord. of right tile
        sta PAD1R.X
        sta PAD2R.X

        lda init_data+1        ;Y-coord of top paddle tiles (same for all)
        sta PAD2L.Y
        sta PAD2M.Y
        sta PAD2R.Y

        lda init_data+2         ;Y-coord of low paddle tiles (same for all)
        sta PAD1L.Y
        sta PAD1M.Y
        sta PAD1R.Y

        lda init_data+3         ;Y-offset of PAD3/4
        sta <PAD.Y.HI            ;also in RAM
        clc
        sta PAD3T.Y             ;top tile
        sta PAD4T.Y
        adc #8
        sta PAD3C.Y             ;center tile
        sta PAD4C.Y
        adc #8
        sta PAD3B.Y             ;bottom tile
        sta PAD4B.Y

        lda init_data+4         ;X-offset of PAD3
        sta PAD3T.X
        sta PAD3C.X
        sta PAD3B.X

        lda init_data+5         ;X-offset of PAD4
        sta PAD4T.X
        sta PAD4C.X
        sta PAD4B.X

        lda init_data+6         ;status for each paddle tile
        sta PAD1L.S
        lda init_data+7
        sta PAD1M.S
        lda init_data+8
        sta PAD1R.S
        lda init_data+9
        sta PAD2L.S
        lda init_data+10
        sta PAD2M.S
        lda init_data+11
        sta PAD2R.S
        lda init_data+12
        sta PAD3T.S
        lda init_data+13
        sta PAD3C.S
        lda init_data+14
        sta PAD3B.S
        lda init_data+15
        sta PAD4T.S
        lda init_data+16
        sta PAD4C.S
        lda init_data+17
        sta PAD4B.S

        lda init_data+18        ;tile # of left and right sprites of paddles
        sta PAD1L.T
        sta PAD1R.T
        sta PAD2L.T
        sta PAD2R.T
        lda init_data+19        ;tile # of middle sprite of paddles
        sta PAD1M.T
        sta PAD2M.T
        lda init_data+20        ;tile # of top & bottom of PAD3/4
        sta PAD3T.T
        sta PAD3B.T
        sta PAD4T.T
        sta PAD4B.T
        lda init_data+21        ;tile # of central tiles of PAD3/4
        sta PAD3C.T
        sta PAD4C.T

        lda init_data+22        ;Y-coord of ball sprite
        sta <BALL.Y.HI           ;is also used in RAM
        sta BALL.Y
        lda init_data+23        ;Tile # of ball sprite
        sta BALL.T
        lda init_data+24        ;status of ball sprite
        sta BALL.S
        lda init_data+25        ;X-coord of ball sprite
        sta <BALL.X.HI           ;is also used in RAM
        sta BALL.X

        clc
        lda init_data+26        ;top edge of japanese title
        sta T.M1.Y
        sta T.M2.Y
        sta T.M3.Y
        adc #8
        sta T.SU.Y
        sta T.I.Y
        sta T.RO.Y
        sta T.HO.Y
        sta T.N.Y
        sta T.KU.Y

        clc
        lda init_data+27        ;left edge of japanese title
        sta T.M1.X
        sta T.SU.X
        adc #8
        sta T.I.X
        adc #8
        sta T.RO.X
        adc #8
        sta T.M2.X
        sta T.HO.X
        adc #8
        sta T.N.X
        adc #8
        sta T.M3.X
        sta T.KU.X

        lda init_data+28        ;status bytes of all japanese title sprites
        sta T.SU.S
        sta T.I.S
        sta T.RO.S
        sta T.HO.S
        sta T.N.S
        sta T.KU.S
        sta T.M1.S
        sta T.M2.S
        sta T.M3.S

        lda init_data+29        ;left edge of scores
        sta S1.X
        clc
        adc #8
        sta S2.X
        adc #8
        sta S3.X
        lda init_data+30        ;top edge
        sta S1.Y
        sta S2.Y
        sta S3.Y
        lda init_data+31        ;status
        sta S1.S
        sta S2.S
        sta S3.S

        ldy #6
        sty T.SU.T
        iny
        sty T.I.T
        iny
        sty T.RO.T
        iny
        sty T.HO.T
        iny
        sty T.N.T
        iny
        sty T.KU.T
        iny
        sty T.M1.T
        sty T.M3.T
        iny
        sty T.M2.T
        ;iny

        lda #0                  ;store zero for...
        sta <PAD.X.LO            ;paddle X coord, low
        sta <PAD.XM.LO           ;paddle X movement
        sta <PAD.XM.HI
        sta <BALL.X.LO           ;ball X/Y coords, low
        sta <BALL.Y.LO
        sta <BALL.XM.LO          ;ball X/Y movement, low
        sta <BALL.YM.LO
        sta <SCORE.LO           ;score
        sta <SCORE.MID
        sta <SCORE.HI

        ldy #$10                ;set score tiles (the score display is updated
        sty S1.T                ;by changing the tiles themselves)
        iny
        sty S2.T
        iny
        sty S3.T

        lda #$ff                ;set ball movement to $FF00,$0100 (moving
        sta <BALL.XM.HI          ;down and left)
        lda #1
        sta <BALL.YM.HI
        ;sta <BALL.XM.HI
        ;sta <BALL.XM.LO

        lda #7                  ;that's it. get the page #...
        sta $4014               ;...and write to Sprite DMA register

        rts

        ;what a merry little chore that was. and that's just the inits!

tiles:                  ;copy embedded tiles (except numbers) to VRAM
        lda #0          ;set VRAM row for first row of tiles ($0000)
        sta $2006
        lda #$10        ;skip the first tile (blank space)
        sta $2006
        ldy #0          ;reset indexer
tiles1:
        lda tileset,y
        sta $2007
        iny
        cpy #$d0        ;$D number of tiles x $10 bytes per tile
        bne tiles1
        rts

ppuinit:
        lda #%00000000
        sta $2000
        lda #%00010100          ;sprites on (no clipping), BG off, no colour
        sta $2001
        rts

body: ;------------------------------------------==[ MAIN BODY ROUTINE ]==----
                                ;below: task, followed by (example)
        jsr readjoy             ;get joystick input (set RIGHT.READ)
        jsr process             ;apply input to movement (set PAD.XM.HI)
        jsr move                ;apply movement to locations (set PAD.X.HI)
        jsr move2               ;apply locations to sprite data in RAM
                                                               ;(set PAD1L.X)
        jsr stall               ;slow down program
        jsr wait_vblank         ;wait for VBlank...
        lda #7                  ;and copy all that to Sprite RAM
        sta $4014
        jmp body                ;repeat

stall:
        ldy #16                 ;increase this to slow down program more
stall1:
        ldx #0
stall2:
        dex
        bne stall2
        dey
        bne stall1
        rts

readjoy:                        ;get input
        ldx #1                  ;strobe-reset joystick register to prepare
        stx $4016               ;for reading
        dex
        stx $4016
do_readjoy:
        lda $4016
        sta <A.READ,x           ;A.READ is the start of the joy data array
        inx
        cpx #8
        bne do_readjoy
        rts

process:                        ;input -> movement
        lda <SEL.READ            ;check if Select+Start (reset)
        beq process1
        lda <STA.READ
        beq process1
        lda #0                  ;if it got here, user pressed Select+Start...
        sta $2001               ;shut off screen display...
        jmp [$FFFC]             ;then goto @ reset vector
process1:
;        lda <B.READ
;        beq process2
;        inc <ACCEL.FACT
;        ldy <ACCEL.FACT
;        bne process2
;        ldy #$ff
;        sty <ACCEL.FACT
        lda <B.READ
        beq process2
        clc
        lda <ACCEL.FACT
        adc #2                  ;if B pressed, increase acceleration factor
        sta <ACCEL.FACT
        bcc process2            ;make sure it didn't wrap around back to $00
        lda #$ff
        sta <ACCEL.FACT
process2:
        lda <B.READ
        bne process3
        dec <ACCEL.FACT
        ldy <ACCEL.FACT
        cpy #$ff
        bne process3
        ldy #0
        sty <ACCEL.FACT
;        lda <B.READ
;        bne process3
;        lda <ACCEL.FACT
;        sec
;        sbc #2                  ;if B not pressed, decrease accel. factor
;        sta <ACCEL.FACT
;        bcs process3            ;make sure it didn't wrap back around to $FF
;        lda #0
;        sta <ACCEL.FACT
process3:
        lda #0                  ;set movement to $0000 beforehand to clear any
        sta <PAD.XM.HI           ;previous movement
        sta <PAD.XM.LO
        lda <LEFT.READ
        eor <RIGHT.READ
        beq no_hmove             ;if Left+Right, they cancel each other out
        lda #1
        sta <PAD.XM.HI
        lda <ACCEL.FACT
        sta <PAD.XM.LO
no_hmove:
        lda #0                  ;do same for Y movement
        sta <PAD.YM.HI
        sta <PAD.YM.LO
        lda <UP.READ
        eor <DOWN.READ
        beq no_vmove
        lda #1
        sta <PAD.YM.HI
        lda <ACCEL.FACT
        sta <PAD.YM.LO
no_vmove:
        rts

move:                           ;movement -> locations (RAM)
        lda <PAD.XM.HI
        beq no_left_read        ;will skip calculations if no movement
        lda <RIGHT.READ
        beq no_right_read
        clc
        lda <PAD.X.LO           ;move right: add movement to location
        adc <PAD.XM.LO
        sta <PAD.X.LO
        lda <PAD.X.HI
        adc <PAD.XM.HI
        sta <PAD.X.HI
no_right_read:
        lda <LEFT.READ
        beq no_left_read
        sec
        lda <PAD.X.LO           ;move left: subtract movement from location
        sbc <PAD.XM.LO
        sta <PAD.X.LO
        lda <PAD.X.HI
        sbc <PAD.XM.HI
        sta <PAD.X.HI
no_left_read:                   ;now do the same for up/down read
        lda <PAD.YM.HI
        beq no_up_read
        lda <DOWN.READ
        beq no_down_read
        clc
        lda <PAD.Y.LO
        adc <PAD.YM.LO
        sta <PAD.Y.LO
        lda <PAD.Y.HI
        adc <PAD.YM.HI
        sta <PAD.Y.HI
no_down_read:
        lda <UP.READ
        beq no_up_read
        sec
        lda <PAD.Y.LO
        sbc <PAD.YM.LO
        sta <PAD.Y.LO
        lda <PAD.Y.HI
        sbc <PAD.YM.HI
        sta <PAD.Y.HI
no_up_read:
        jsr pad_boundary        ;check for boundaries

        clc                     ;add ball movements and shit
        lda <BALL.X.LO          ;(there is no "is ball moving left or right"
        adc <BALL.XM.LO         ;or up/down variable; the ball's movement is
        sta <BALL.X.LO          ;a signed 16-bit integer (ie. $FF00 = $-0100))
        lda <BALL.X.HI
        adc <BALL.XM.HI
        sta <BALL.X.HI
        clc
        lda <BALL.Y.LO
        adc <BALL.YM.LO
        sta <BALL.Y.LO
        lda <BALL.Y.HI
        adc <BALL.YM.HI
        sta <BALL.Y.HI
        jsr ball_boundary       ;check for boundaries
        rts                     ;that was easy enough...

pad_boundary:                   ;OK if 8 <= X < 224
        lda #9                  ;evaluate (8 <= X) and branch on result true
        clc
        sbc <PAD.X.HI
        bcc not_pad_onleft
        lda #9                  ;if X crosses left boundary, set X position
        sta <PAD.X.HI            ;to $0800
        lda #0
        sta <PAD.X.LO
not_pad_onleft:
        lda #219                ;evaluate (X >= 224) and branch on res. false
        clc 
        sbc <PAD.X.HI
        bcs not_pad_onright
        lda #218                ;if X crosses right boundary, set X position
        sta <PAD.X.HI            ;to $DFFF ($DF = 223)
        lda #$ff
        sta <PAD.X.LO
not_pad_onright:
        lda #22
        clc
        sbc <PAD.Y.HI
        bcc not_pad_ontop
        lda #22
        sta <PAD.Y.HI
        lda #0
        sta <PAD.Y.LO
not_pad_ontop:
        lda #187
        clc
        sbc <PAD.Y.HI
        bcs not_pad_onfloor
        lda #186
        sta <PAD.Y.HI
        lda #$ff
        sta <PAD.Y.LO
not_pad_onfloor:
        rts

ball_boundary:                  ;OK if ((8<=X<240) && (8<=Y<224))
        lda #8                  ;same as pad_boundary except checks Y too, and
        clc                     ;will change movement as necessary
        sbc <BALL.X.HI
        bcc not_ball_onleft
        lda #8              ; <- sets ball location as being ON the boundary
        sta <BALL.X.HI
        lda #0
        sta <BALL.X.LO
        sta <BALL.XM.LO     ; <- then sets X movement to $0100 (moving right)
        lda #1
        sta <BALL.XM.HI
        jmp side_collision      ;check for side collisions
not_ball_onleft:                ;(that routine ends in RTS, BTW)
        lda #235
        clc
        sbc <BALL.X.HI
        bcs not_ball_onright
        lda #234            ; <- sets location as being barely ON right bound.
        sta <BALL.X.HI
        lda #$ff            ; <- then sets X movement to $FF00 (moving left)
        sta <BALL.X.LO
        sta <BALL.XM.HI
        lda #0
        sta <BALL.XM.LO
        jmp side_collision
not_ball_onright:               ;identical as above but for Y instead (and
        lda #21                 ;the actual boundaries are a bit different)
        clc
        sbc <BALL.Y.HI
        bcc not_ball_ontop
        lda #22
        sta <BALL.Y.HI
        lda #0
        sta <BALL.Y.LO
        sta <BALL.YM.LO
        lda #1
        sta <BALL.YM.HI
        jmp collision_detection ;checks to see if it hit the paddle
not_ball_ontop:
        lda #204
        clc
        sbc <BALL.Y.HI
        bcs not_ball_onfloor
        lda #203
        sta <BALL.Y.HI
        lda #$ff
        sta <BALL.Y.LO
        sta <BALL.YM.HI
        lda #0
        sta <BALL.YM.LO
        jmp collision_detection ;NOTE: that routine also ends in RTS
not_ball_onfloor:
        rts
                        ;          +------------- paddle X location - 4
                        ;          |     +------- ball X location
                        ;          |     |   +--- paddle X location + 20
collision_detection:    ;hit if ((PX-4)<BX<(PX+20))
                        ;modified P in TEMP1 (LO) - TEMP2 (HI)
        sec             ;evaluates (BALL > (PAD-4)) and branches on true
        lda <PAD.X.LO
        sta <TEMP1
        lda <PAD.X.HI
        sbc #4
        sta <TEMP2
        sec
        lda <TEMP1
        sbc <BALL.X.LO
        lda <TEMP2
        sbc <BALL.X.HI
        bcc hitleft     ;branches if check of left side returns hit
        jmp missed      ;if this is reached, ball missed: you suck!
hitleft:
        clc             ;evaluates ((PAD+20) > BALL), branches on true
        lda <PAD.X.LO
        sta <TEMP1
        lda <PAD.X.HI
        adc #20
        sta <TEMP2
        sec
        lda <TEMP1
        sbc <BALL.X.LO
        lda <TEMP2
        sbc <BALL.X.HI
        bcs hitright    ;branches if check of right side returns hit
        jmp missed      ;you missed! go home
hitright:
        jmp inc_counter ;you got lucky: add 1 to score

side_collision:
        sec
        lda <PAD.Y.LO
        sta <TEMP1
        lda <PAD.Y.HI
        sbc #4
        sta <TEMP2
        sec
        lda <TEMP1
        sbc <BALL.Y.LO
        lda <TEMP2
        sbc <BALL.Y.HI
        bcc hittop
        jmp missed
hittop:
        clc
        lda <PAD.Y.LO
        sta <TEMP1
        lda <PAD.Y.HI
        adc #20
        sta <TEMP2
        sec
        lda <TEMP1
        sbc <BALL.Y.LO
        lda <TEMP2
        sbc <BALL.Y.HI
        ;bcs hitfloor
        bcs inc_counter
        jmp missed
;hitfloor:
        ;jmp inc_counter        ;don't need to JMP if it's right here!

inc_counter:                    ;increases the score
        inc <SCORE.LO
        ldy <SCORE.LO
        cpy #10
        bne no_s_carry
        lda #0
        sta <SCORE.LO

        inc <SCORE.MID
        ldy <SCORE.MID
        cpy #10
        bne no_s_carry
        lda #0
        sta <SCORE.MID
        sta <SCORE.LO

        inc <SCORE.HI
        ldy <SCORE.HI
        cpy #10
        bne no_s_carry
        sta <SCORE.HI
        sta <SCORE.MID
        sta <SCORE.LO
no_s_carry:
        rts

move2:                          ;locations(RAM) -> sprite data (7-page)
        clc                     ;this will be easy, just use the high bytes;
        lda <PAD.X.HI            ;this is how I planned to do it anyway. ani-
        sta PAD1L.X             ;mation might be choppy but it'll effectively
        sta PAD2L.X             ;let me use 16-bit numbers for all velocity
        adc #8                  ;and position variables; just use the high-
        sta PAD1M.X             ;order byte for the 8-bit screen coordinates.
        sta PAD2M.X
        adc #8
        sta PAD1R.X
        sta PAD2R.X

        clc
        lda <PAD.Y.HI
        sta PAD3T.Y
        sta PAD4T.Y
        adc #8
        sta PAD3C.Y
        sta PAD4C.Y
        adc #8
        sta PAD3B.Y
        sta PAD4B.Y

        lda <BALL.X.HI
        sta BALL.X
        lda <BALL.Y.HI
        sta BALL.Y

        lda <SCORE.LO           ;copies score numbers into tiles in VRAM
        sta <TEMP1              ;it does this by taking the values in SCORE.LO
        lda #1                  ;through SCORE.HI, copying them to TEMP1, and
        sta $2006               ;calling the do_score routine (ie. passing
        lda #$20                ;them along as an argument), which is what
        sta $2006               ;actually copies the tiles (it uses the
        jsr do_score            ;pointer table of the number tiles; it cal-
        lda <SCORE.MID          ;culates which pointer to use by multiplying
        sta <TEMP1              ;TEMP1 (ie. the argument) by 2 (because every
        lda #1                  ;pointer is two bytes, eg. the FIFTH pointer
        sta $2006               ;starts at the TENTH byte) (use the ASL in-
        lda #$10                ;struction to multiply by 2); it copies that
        sta $2006               ;pointer to POINT.LO and POINT.HI, then uses
        jsr do_score            ;LDA (POINT.LO),Y to read the data through the
        lda <SCORE.HI           ;pointer. Hehe.
        sta <TEMP1
        lda #1
        sta $2006
        lda #0
        sta $2006
        jsr do_score

        rts

do_score:
        lda <TEMP1              ;get number # to copy
        asl a                   ;double it (each pointer is TWO bytes)
        tay                     ;place in index
        lda num_pointers,y      ;setup the pointer in RAM
        sta <POINT.LO
        lda num_pointers+1,y
        sta <POINT.HI
        ldy #0                  ;reset index
do_score2:
        lda [POINT.LO],y        ;reads from the addr. at pointer and adds Y
        sta $2007
        iny
        cpy #$10
        bne do_score2
        rts

missed:                 ;this will stall until you press start, then reset
        jsr readjoy
        lda <STA.READ
        beq missed
        lda #0
        sta $2001       ;turn off screen display
        jmp [$FFFC]

;if you're modifying this, you can comment-out the above set of commands (in
;the "missed" routine) and un-comment-out the ones below, for play-testing
;purposes (it will just change the ball's colours when you miss).

;        ldx #$ff       ;sets the stack to the top (again)
;        txs
;        lda #%00000011 ;will use OR to force-set the lower 2 bits of the ball
;        ora BALL.S     ;status, to use the blue colour set instead of red
;        sta BALL.S
;        jmp body       ;then resumes normal execution

int:
        rti             ;dummy label: just returns

;that's it for now...

        .bank 1
        .org $fffa
        .dw int,main,int        ;define the 6502 vector table

;THE END


;                              ----==[#]==----
;                               29 MARCH 2001

;I am the Bullgod
;I am free

;I'm gonna get you
;I see through you
;I'm gonna get you

;I'm like a train I roll hard lettin' off much steam
;in the cutoff flannel and the dusty jeans baby
;I never was cool with James Dean
;but I be hangin' tough with my man Jim Bean
;I swing low like a chimp
;back in '86 man I was seein' a shrink
;but now I'm humble and I can only think
;about New Orleans and those jumbo shrimp
;so ask no questions and I tell no lies
;I got big ol' pupils and bloodshot eyes
;I'm on the brink if you know what I mean
;and a 12-step program couldn't keep me clean
;'cause I'm the bullgod you understand
;the illegitimate son of man
;the T-O-P to the D-O-G or the P-O-T to the G-O-D
;and I'm trippin'
;said I'm trippin'

;I am the Bullgod
;I am free
;I'm gonna get you
;I see through you
;I'm gonna get you

;a lot of people poke fun and that's alright
;but when I start pokin' back they get all uptight, huh
;you can't cap with the master, son
;so sit yo' ass down 'fore I blast ya one
;'cause I'm so greasy you can call me mud
;and I feel a little hank runnin' through my blood
;I'm gonna flood the world with my twisted thoughts
;you can bid all day but I can't be bought, huh
;break it up let's tie one on
;I gotta get set to go and cut the lawn
;so I grab my walkman but before I cut
;I go behind the garage and fire it up
;'cause I'm the bullgod you understand
;the illegitimate son of man
;the T-O-P to the D-O-G or the P-O-T to the G-O-D
;and I'm trippin'
;said I'm trippin'

;I am the Bullgod
;I am free
;...I'm forsaken

;yeah, yeah, yeah... come on, get 'em up!

;I am the Bullgod
;I am free

;I get a feelin' of peace from my low slow high
;as I sit in my chair and watch life go by
;these thoughts I have I can't more ___ (?)
;through the forests of my mind they're all past-tense
;born and raised in the outer lands
;and at times you can say I'm out of hand
;I'm in a band of gypsies we're on the run
;every time that paper hits my tongue
;and sometimes I seem so high
;with my veins are poppin' and I more ___ (?)
;I am the Bullgod you understand
;and here in my head is my master plan

;I'm gonna get you
;I see through you
;I'm gonna get you
;I see through you

;                       ---"I am the Bullgod"
;                                by Kid Rock



;we do what we say
;you say what we do
